#PB_DBL_EPS        = 2.2204460492503131e-016   ; smallest such that 1.0+#PB_DBL_EPS != 1.0
#PB_EPS_QUAD       = 4.9303806576313238e-032
#PB_EPS_ROOT       = 1.49146253473e-8

Prototype.d Pb_DblFkt(Wert.d)

Macro M_ABS(w)
  Abs(w)
EndMacro

Macro M_SIGN2(x,y)
  MathSign2(x,y)
EndMacro

Macro M_SIGN(w)
  MathSign(w)
EndMacro

Procedure.d MathSign2 (x.d,y.d)
  If y < 0.0
    ProcedureReturn -M_ABS(x)
  ElseIf y > 0.0
    ProcedureReturn M_ABS(x)
  EndIf
  ProcedureReturn 0.0
EndProcedure

Procedure.d MathSign (w.d)
  If w < 0.0
    ProcedureReturn -1.0
  ElseIf w > 0.0
    ProcedureReturn 1.0
  EndIf
  ProcedureReturn 0.0
EndProcedure

Enumeration
  #Window_Num
EndEnumeration

;- Gadget Constants
;
Enumeration
  #Image_Ausgabe
  #Text_Form1
  #Text_Form2
  #String_Form1
  #String_Form2
  #ListIcon_Schnittpunkt
  #Text_Xmin
  #Text_Xmax
  #Text_Ymin
  #Text_Ymax
  #String_Xmin
  #String_Xmax
  #String_Ymin
  #String_Ymax
  #Button_Berechnen
  #Button_Beenden
  #Button_Info
  #Image0
EndEnumeration

;- StatusBar Constants
;
Enumeration
  #StatusBar_Fenster
EndEnumeration


; Rekursiv absteigender Parser.
; Calc() ist die einzige aufrufbare Routine, ihr muss ein String
; bergeben werden, der einen mathematischen Ausdruck enthlt.
; Es wird wieder ein String zurckgeliefert. Dieser ist entweder
; das Ergebnis (erstes Zeichen ist eine Ziffer) oder eine Fehler-
; meldung (erstes Zeichen ist keine Ziffer).
; -- getestet mit PureBasic 4.20
; -- Little John, 27.7.2008

#PB_DBL_EPS        = 2.2204460492503131e-016   ; smallest such that 1.0+#PB_DBL_EPS != 1.0
#PB_QUAD_MAX       = 9223372036854775807       ; maximum signed long long int value

#BIT_0             =  1
#BIT_1             =  2
#BIT_2             =  4
#BIT_3             =  8
#BIT_4             = 16

CompilerIf #PB_Compiler_Version = 420
  Macro List
  EndMacro
  Macro Array
  EndMacro
  Macro ListSize
    CountList
  EndMacro
  Macro PokeI
    PokeL
  EndMacro
CompilerEndIf

EnableExplicit

;-- Token types
Enumeration
   #Calc_Unknown
   #Calc_Operator
   #Calc_DecNumber
   #Calc_HexNumber
   #Calc_Identifier
EndEnumeration

;-- Token definitions
#Calc_OpRel = "<=<>="
#Calc_OpChars = #Calc_OpRel + "+-*/:^),"
#Calc_Letters = "abcdefghijklmnopqrstuvwxyz"
#Calc_DecDigits = "0123456789"
#Calc_HexDigits = #Calc_DecDigits + "abcdef"
#Calc_AlphaNum = #Calc_Letters + #Calc_DecDigits

;-- Constants
#E = 2.7182818284590452

;-- Error messages
Enumeration
  #CalcErr_BadSyntax = 1               ; "Syntaxfehler"
  #CalcErr_MissingOperator             ; "Operator fehlt"
  #CalcErr_NoClosingBracket            ; ") fehlt"
  #CalcErr_NoOpeningBracket            ; "( fehlt"
  #CalcErr_UnknownOperator             ; "Unbekannter Operator: "
  #CalcErr_UnknownFunction             ; "Unbekannte Funktion: "
  #CalcErr_UnknownConstant             ; "Unbekannte Konstante: "
  #CalcErr_InvalidDecNumber            ; "Ungltige Dezimalzahl"
  #CalcErr_InvalidHexNumber            ; "Ungltige Hexadezimalzahl"
  #CalcErr_ParameterMismatch           ; "(): Falsche Parameteranzahl"
  #CalcErr_NegativeBase                ; "Negative Basis mit gebrochenem Exponenten"
  #CalcErr_DivisionByZero              ; "Division durch Null"
  #CalcErr_PowerUndefined              ; "Null hoch Null ist nicht definiert"
  #CalcErr_Sqr                         ; "Quadratwurzel aus negativer Zahl"
  #CalcErr_Log                         ; "Logarithmus aus Zahl <= 0"
  #CalcErr_LogBase                     ; "Logarithmus zu Basis <= 0 oder 1"          
  #CalcErr_OutOfPrecition              ; "(): Argument zu gro"
  #CalcErr_Tan                         ; "Tan(): Argument = Pi/2 + i * Pi"
  #CalcErr_ACosASin                    ; "(): Abs(Argument) > 1" 
EndEnumeration

Structure Calc_Tokens
   Str.s
   Typ.l
EndStructure

Declare.d Calc_Expression(Array TokenArray.Calc_Tokens(1))
Declare.i Calc_ErrorI(iErr.i)
Declare.s Calc_ErrorS(sErr.s)
Declare.s Calc_GetErrorText(iNr.i)
Declare.s Calc_NextToken(*Tokentyp.i,expr.s)
Declare.i Calc_Scan (expr.s,Array TokenArray.Calc_Tokens(1))
Declare.i Calc_GetToken (iAdd.i,idx.l)
Declare.d Calc_Min(List Args.d())
Declare.d Calc_Max(List Args.d())
Declare.d Calc_Sum(List Args.d())
Declare.d Calc_Func(name.s,List Args.d())
Declare.d Calc_Const(name.s)
Declare.d Calc_Arith(op.s, v.d, r.d)
Declare.d Calc_Factor(Array TokenArray.Calc_Tokens(1))
Declare.d Calc_Power(Array TokenArray.Calc_Tokens(1))
Declare.d Calc_Term(Array TokenArray.Calc_Tokens(1))
Declare.d Calc_Sign(Array TokenArray.Calc_Tokens(1))
Declare.d Calc_SingleExpression(Array TokenArray.Calc_Tokens(1))
Declare.d Calc_Expression(Array TokenArray.Calc_Tokens(1))

;-----------------------------------------------------------------------
Procedure.i Calc_ErrorI(iErr.i=-1)

   Static Error.i = 0
   If iErr <> -1
     Error = iErr
     ProcedureReturn 0
   EndIf
   
   ProcedureReturn Error   
EndProcedure

Procedure.s Calc_ErrorS(sErr.s="")

   Static Error.s = ""
   If sErr <> ""
     Error = sErr
     ProcedureReturn ""
   EndIf
   
   ProcedureReturn Error   
EndProcedure

Procedure.s Calc_GetErrorText(iNr.i)
  Select iNr
    Case #CalcErr_BadSyntax               : ProcedureReturn "Syntaxfehler"
    Case #CalcErr_MissingOperator         : ProcedureReturn "Operator fehlt"
    Case #CalcErr_NoClosingBracket        : ProcedureReturn ") fehlt"
    Case #CalcErr_NoOpeningBracket        : ProcedureReturn "( fehlt"
    Case #CalcErr_UnknownOperator         : ProcedureReturn "Unbekannter Operator: "+Calc_ErrorS()
    Case #CalcErr_UnknownFunction         : ProcedureReturn "Unbekannte Funktion: "+Calc_ErrorS()
    Case #CalcErr_UnknownConstant         : ProcedureReturn "Unbekannte Konstante: "+Calc_ErrorS()
    Case #CalcErr_InvalidDecNumber        : ProcedureReturn "Ungltige Dezimalzahl"
    Case #CalcErr_InvalidHexNumber        : ProcedureReturn "Ungltige Hexadezimalzahl"
    Case #CalcErr_ParameterMismatch       : ProcedureReturn Calc_ErrorS()+"(): Falsche Parameteranzahl"
    Case #CalcErr_NegativeBase            : ProcedureReturn "Negative Basis mit gebrochenem Exponenten"
    Case #CalcErr_DivisionByZero          : ProcedureReturn "Division durch Null"
    Case #CalcErr_PowerUndefined          : ProcedureReturn "Null hoch Null ist nicht definiert"
    Case #CalcErr_Sqr                     : ProcedureReturn "Quadratwurzel aus negativer Zahl"
    Case #CalcErr_Log                     : ProcedureReturn "Logarithmus aus Zahl <= 0"
    Case #CalcErr_LogBase                 : ProcedureReturn "Logarithmus zu Basis <= 0 oder 1"          
    Case #CalcErr_OutOfPrecition          : ProcedureReturn Calc_ErrorS()+"(): Argument zu gro"
    Case #CalcErr_Tan                     : ProcedureReturn "Tan(): Argument = Pi/2 + i * Pi"
    Case #CalcErr_ACosASin                : ProcedureReturn Calc_ErrorS()+"(): Abs(Argument) > 1" 
  EndSelect       
EndProcedure                 

Procedure.s Calc_NextToken (*Tokentyp.i,expr.s="")
   ; in : expr     : zu berechnender Ausdruck
   ; out: LokalToken    : aktueller LokalToken
   ;      LokalTokenType: Typ des aktuellen Tokens
   Static formula.s, posn.i
   Protected LokalTokenType.i
   Protected LokalToken.s

   If expr <> ""      ; nur beim ersten Aufruf
      formula = expr
      posn = 1
   EndIf

   While posn <= Len(formula) And Mid(formula, posn, 1) = " "
      posn + 1
   Wend

   LokalToken = Mid(formula, posn, 1)
   posn + 1

   If FindString(#Calc_OpChars, LokalToken, 1)
      If FindString(#Calc_OpRel, LokalToken, 1) <> 0 And posn <= Len(formula) And FindString(#Calc_OpRel, Mid(formula, posn, 1), 1) <> 0
         LokalToken + Mid(formula, posn, 1)
         posn + 1
      EndIf
      LokalTokenType = #Calc_Operator

   ElseIf FindString(#Calc_Letters, LokalToken, 1)
      If (LokalToken = "e" And FindString(#Calc_DecDigits, Mid(formula, posn, 1), 1) = 0) Or LokalToken <> "e" ; wegen Exponentialform z.B. 2e2
         While posn <= Len(formula) And FindString(#Calc_AlphaNum, Mid(formula, posn, 1), 1) <> 0
            LokalToken + Mid(formula, posn, 1)
            posn + 1
         Wend
      EndIf
      LokalTokenType = #Calc_Identifier

   ElseIf FindString(#Calc_DecDigits, LokalToken, 1)
      While posn <= Len(formula) And FindString(#Calc_DecDigits, Mid(formula, posn, 1), 1) <> 0
         LokalToken + Mid(formula, posn, 1)
         posn + 1
      Wend
      LokalTokenType = #Calc_DecNumber

   ElseIf LokalToken = "$"
      While posn <= Len(formula) And FindString(#Calc_HexDigits, Mid(formula, posn, 1), 1) <> 0
         LokalToken + Mid(formula, posn, 1)
         posn + 1
      Wend
      LokalTokenType = #Calc_HexNumber

   Else
      LokalTokenType = #Calc_Unknown
   EndIf
   PokeI(*Tokentyp,LokalTokenType)
   ProcedureReturn LokalToken
EndProcedure

Procedure.i Calc_Scan (expr.s,Array TokenArray.Calc_Tokens(1))
   ;-- kompletten Ausdruck vorab in die einzelnen Token aufteilen,
   ;   und diese in TokenArray() speichern.
   Protected index.i,LokalTokenType.i,LokalToken.s

   ReDim TokenArray.Calc_Tokens(Len(expr)*2)  ; so ist das Array immer gro genug

   index = 0
   LokalToken = Calc_NextToken(@LokalTokenType,expr)
   While LokalToken <> ""
      TokenArray(index)\Str = LokalToken
      TokenArray(index)\Typ = LokalTokenType
      index + 1
      LokalToken = Calc_NextToken(@LokalTokenType)
   Wend
EndProcedure

Procedure.i Calc_GetToken (iAdd.i,idx.l=-1)

   Static index

   If idx <> -1        ; nur beim ersten Aufruf
      index = idx
   EndIf

   index + iAdd
   ProcedureReturn index
EndProcedure

;-----------------------------------------------------------------------
;-- selbst definierte Funktionen

Procedure.d Calc_Min (List Args.d())
   ; in : verknpfte Liste aus beliebig vielen reellen Zahlen
   ; out: kleinste Zahl in der Liste
   Protected ret.d

   ret = Args()
   While NextElement(Args())
      If ret > Args()
         ret = Args()
      EndIf
   Wend
   ProcedureReturn ret
EndProcedure

Procedure.d Calc_Max (List Args.d())
   ; in : verknpfte Liste aus beliebig vielen reellen Zahlen
   ; out: grte Zahl in der Liste
   Protected ret.d

   ret = Args()
   While NextElement(Args())
      If ret < Args()
         ret = Args()
      EndIf
   Wend
   ProcedureReturn ret
EndProcedure

Procedure.d Calc_Sum (List Args.d())
   ; in : verknpfte Liste aus beliebig vielen reellen Zahlen
   ; out: Summe aller Zahlen in der Liste
   Protected ret.d

   ret = Args()
   While NextElement(Args())
      ret + Args()
   Wend
   ProcedureReturn ret
EndProcedure

;-----------------------------------------------------------------------

Procedure.d Calc_Func (name.s,List Args.d())
   ; Hier werden alle Funktionen aufgerufen (egal ob sie in PureBasic
   ; eingebaut oder selbst definiert sind).
   ; Eine leere Argumentliste erzeugt sowieso einen Syntaxfehler,
   ; daher braucht das hier nicht berprft zu werden.
   Protected v.d = 0

   SelectElement(Args(), 0)

   Select name
      Case "sqr","sqrt"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         ElseIf Args() < -2*#PB_DBL_EPS
            Calc_ErrorI(#CalcErr_Sqr)
         ElseIf Args() < 0.0
            v = 0.0
         Else            
            v = Sqr(Args())
         EndIf

      Case "log"
         If ListSize(Args()) > 2
            Calc_ErrorS(name) 
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         ElseIf Args() <= 0
            Calc_ErrorI(#CalcErr_Log)
         Else
            v = Log(Args())                  ; Die Standard-Basis ist e.
            If ListSize(Args()) = 2         ; Wenn eine andere Basis angegeben ist.
               NextElement(Args())
               If Args() <= 0 Or Args() = 1
                  Calc_ErrorI(#CalcErr_LogBase)
               Else
                  v / Log(Args())
               EndIf
            EndIf
         EndIf
         
      Case "sin"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            v = Args()
            If v*#PB_DBL_EPS > 2*#PI
              Calc_ErrorS(name)
              Calc_ErrorI(#CalcErr_OutOfPrecition) 
            EndIf
            v = Sin(v)
         EndIf

      Case "cos"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            If Args()*#PB_DBL_EPS > 2*#PI
              Calc_ErrorS(name)
              Calc_ErrorI(#CalcErr_OutOfPrecition) 
            EndIf
            v = Cos(Args())
         EndIf

      Case "tan"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            If Args()*#PB_DBL_EPS > 2*#PI
              Calc_ErrorS(name)
              Calc_ErrorI(#CalcErr_OutOfPrecition) 
            Else
              Protected ganz.i
              
              ganz = Int(2*Args()/#PI)
              If (ganz%2) = 1 And ganz*#PI*0.5 = Args()
                Calc_ErrorS(name)
                Calc_ErrorI(#CalcErr_OutOfPrecition)                  
              EndIf
            EndIf
            v = Tan(Args())
         EndIf
         
      Case "atan"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            v = ATan(Args())
         EndIf

      Case "asin"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            If Args() > 1.0+4*#PB_DBL_EPS Or Args() < -1.0-4*#PB_DBL_EPS
              Calc_ErrorS(name)
              Calc_ErrorI(#CalcErr_ACosASin)
              If Args() > 1.0
                Args() = 1.0
              Else
                Args() = -1.0
              EndIf            
            EndIf
            v = ASin(Args())
         EndIf

      Case "acos"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            If Args() > 1.0+4*#PB_DBL_EPS Or Args() < -1.0-4*#PB_DBL_EPS
              Calc_ErrorS(name)
              Calc_ErrorI(#CalcErr_ACosASin)
              If Args() > 1.0
                Args() = 1.0
              Else
                Args() = -1.0
              EndIf            
            EndIf
            v = ACos(Args())
         EndIf

      Case "abs"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            v = Abs(Args())
         EndIf

      Case "int"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            If Args()>#PB_QUAD_MAX Or -Args()>#PB_QUAD_MAX
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_OutOfPrecition)
            EndIf
            v = IntQ(Args())
         EndIf

      Case "grad"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            v = Args()*180./#PI
         EndIf

      Case "rad"
         If ListSize(Args()) <> 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            v = Args()*#PI/180.
         EndIf
      
      Case "pow"
         If ListSize(Args()) <> 2
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else            
            v = Args()
            NextElement(Args())
            v = Pow(v,Args())
         EndIf
         
      Case "sign"
         If ListSize(Args()) > 1
            Calc_ErrorS(name)
            Calc_ErrorI(#CalcErr_ParameterMismatch)
         Else
            If Args() > 0.0
              v = 1.0
            ElseIf Args() < 0.0
              v = -1.0
            Else
              v = 0.0
            EndIf
         EndIf

      Case "min"
         v = Calc_Min(Args())

      Case "max"
         v = Calc_Max(Args())

      Case "sum"
         v = Calc_Sum(Args())

      Default
         Calc_ErrorI(#CalcErr_UnknownFunction)
         Calc_ErrorS(name)
   EndSelect
   ProcedureReturn v
EndProcedure

Procedure.d Calc_Const (name.s)
   Protected v.d

   Select name
      Case "pi"
         v = #PI
      Case "e"
         v = #E
      Default
         Calc_ErrorI(#CalcErr_UnknownConstant)
         Calc_ErrorS(name)
   EndSelect
   ProcedureReturn v
EndProcedure

Procedure.d Calc_Arith (op.s, v.d, r.d)
   If Calc_ErrorI()
      ProcedureReturn v
   EndIf

   Select op
      Case "<"
         If v < r
            v = #True
         Else
            v = #False
         EndIf
      Case ">"
         If v > r
            v = #True
         Else
            v = #False
         EndIf
      Case "="
         If v = r
            v = #True
         Else
            v = #False
         EndIf
      Case "<>"
         If v <> r
            v = #True
         Else
            v = #False
         EndIf
      Case "<="
         If v <= r
            v = #True
         Else
            v = #False
         EndIf
      Case ">="
         If v >= r
            v = #True
         Else
            v = #False
         EndIf
      Case "+"
         v + r
      Case "-"
         v - r
      Case "*"
         v * r
      Case "/" , ":"
         If r = 0
            Calc_ErrorI(#CalcErr_DivisionByZero)
         Else
            v / r
         EndIf
      Case "^"
         If v < 0 And r <> Int(r)
            Calc_ErrorI(#CalcErr_NegativeBase)
         ElseIf v = 0 And r < 0
            Calc_ErrorI(#CalcErr_DivisionByZero)
         ElseIf v = 0 And r = 0
            Calc_ErrorI(#CalcErr_PowerUndefined)
         Else
            v = Pow(v, r)
         EndIf
      Default
         Calc_ErrorI(#CalcErr_UnknownOperator)
         Calc_ErrorS(op)
   EndSelect
   ProcedureReturn v
EndProcedure

;-----------------------------------------------------------------------
;-- rekursive Prozeduren

Procedure.d Calc_Factor(Array TokenArray.Calc_Tokens(1))
   Protected op.s,Token.s
   Protected v.d
   Protected TokenType.i = TokenArray(Calc_GetToken(0))\Typ,Akt.i

   Token = TokenArray(Calc_GetToken(0))\Str
   If TokenType = #Calc_DecNumber
      op = Token
      Akt = Calc_GetToken(1)
      Token = TokenArray(Akt)\Str
      If Token <> "."
         v = Val(op)
      Else
         Akt = Calc_GetToken(1)
         Token = TokenArray(Akt)\Str
         TokenType = TokenArray(Akt)\Typ
         If TokenType = #Calc_DecNumber
            v = ValD(op + "." + Token)
         ElseIf Token = "e"
           Token = TokenArray(Calc_GetToken(1))\Str
           TokenType = TokenArray(Calc_GetToken(0))\Typ
           If TokenType = #Calc_DecNumber
             v * Pow(10.,Val(Token))
           ElseIf Token = "+" Or Token = "-"
             Protected Vz.i=0
             If Token="+"
               Vz = 1
             Else
               Vz = -1
             EndIf
             Token = TokenArray(Calc_GetToken(1))\Str
             TokenType = TokenArray(Calc_GetToken(0))\Typ
             If TokenType = #Calc_DecNumber
               v = Val(op) * Pow(10.,Vz*Val(Token))
             Else
               Calc_ErrorI(#CalcErr_MissingOperator)
             EndIf
           Else
             Calc_ErrorI(#CalcErr_MissingOperator)
           EndIf
         Else
            Calc_ErrorI(#CalcErr_InvalidDecNumber)
         EndIf
         Calc_GetToken(1)
      EndIf

   ElseIf TokenType = #Calc_HexNumber
      If Len(Token) > 1
         v = Val(Token)
      Else
         Calc_ErrorI(#CalcErr_InvalidHexNumber)
      EndIf
      Calc_GetToken(1)

   ElseIf TokenType = #Calc_Identifier
      op = Token
      Token = TokenArray(Calc_GetToken(1))\Str
      If Token = "("
         ;-- Liste der Funktionsargumente lesen
         Protected NewList Args.d()
         Repeat
           Calc_GetToken(1)
           AddElement(Args())
           Args() = Calc_Expression(TokenArray())
           Token = TokenArray(Calc_GetToken(0))\Str
         Until Token <> ","
         If Token <> ")" And Calc_ErrorI() = 0
            Calc_ErrorI(#CalcErr_NoClosingBracket)
         EndIf
         Calc_GetToken(1)
         v = Calc_Func(op,Args())
      Else
         v = Calc_Const(op)
      EndIf

   ElseIf Token = "("
      Calc_GetToken(1)
      v = Calc_Expression(TokenArray())
      Token = TokenArray(Calc_GetToken(0))\Str
      If Token <> ")" And Calc_ErrorI() = 0
         Calc_ErrorI(#CalcErr_NoClosingBracket)
      EndIf
      Calc_GetToken(1)

   ElseIf Token = "+" Or Token = "-"

   Else
      Calc_ErrorI(#CalcErr_BadSyntax)
   EndIf
   Token = TokenArray(Calc_GetToken(0))\Str
   TokenType = TokenArray(Calc_GetToken(0))\Typ
   If Token <> "" And FindString(#Calc_OpChars, Token, 1) = 0 And Calc_ErrorI() = 0
      If Token = "e"
        Token = TokenArray(Calc_GetToken(1))\Str
        TokenType = TokenArray(Calc_GetToken(0))\Typ
        If TokenType = #Calc_DecNumber
          v * Pow(10.,Val(Token))
        ElseIf Token = "+" Or Token = "-"
          Vz=0
          If Token="+"
            Vz = 1
          Else
            Vz = -1
          EndIf
          Token = TokenArray(Calc_GetToken(1))\Str
          TokenType = TokenArray(Calc_GetToken(0))\Typ
          If TokenType = #Calc_DecNumber
            v * Pow(10.,Vz*Val(Token))
          Else
            Calc_ErrorI(#CalcErr_MissingOperator)
          EndIf
        Else
          Calc_ErrorI(#CalcErr_MissingOperator)
        EndIf
        Calc_GetToken(1)
      ElseIf TokenType = #Calc_DecNumber Or TokenType = #Calc_HexNumber Or TokenType = #Calc_Identifier Or Token = "("
         Calc_ErrorI(#CalcErr_MissingOperator)
      Else
         Calc_ErrorI(#CalcErr_UnknownOperator)
         Calc_ErrorS(Token)
      EndIf
   EndIf

   ProcedureReturn v
EndProcedure

Procedure.d Calc_Power(Array TokenArray.Calc_Tokens(1))
   Protected v.d, r.d
   Protected Token.s

   v = Calc_Factor(TokenArray())
   Token = TokenArray(Calc_GetToken(0))\Str
   If Token = "^"
      Calc_GetToken(1)
      r = Calc_Power(TokenArray())
      v = Calc_Arith("^", v, r)
   EndIf
   ProcedureReturn v
EndProcedure

Procedure.d Calc_Term(Array TokenArray.Calc_Tokens(1))
   Protected op.s
   Protected v.d, r.d

   v = Calc_Power(TokenArray())
   op = TokenArray(Calc_GetToken(0))\Str
   While (op = "*" Or op = "/" Or op = ":")
      Calc_GetToken(1)
      r = Calc_Sign(TokenArray())
      v = Calc_Arith(op, v, r)
      op = TokenArray(Calc_GetToken(0))\Str
   Wend
   ProcedureReturn v
EndProcedure

Procedure.d Calc_Sign(Array TokenArray.Calc_Tokens(1))
   Protected op.s
   Protected v.d
   Protected Token.s

   op = ""
   Token = TokenArray(Calc_GetToken(0))\Str
   If Token = "+" Or Token = "-"
      op = Token
      Calc_GetToken(1)
   EndIf
   v = Calc_Term(TokenArray())
   If op = "-"
      v = -v
   EndIf
   ProcedureReturn v
EndProcedure

Procedure.d Calc_SingleExpression(Array TokenArray.Calc_Tokens(1))
   Protected op.s
   Protected v.d, r.d

   v = Calc_Sign(TokenArray())
   op = TokenArray(Calc_GetToken(0))\Str
   While (op = "+" Or op = "-")
      Calc_GetToken(1)
      r = Calc_Term(TokenArray())
      v = Calc_Arith(op, v, r)
      op = TokenArray(Calc_GetToken(0))\Str
   Wend
   ProcedureReturn v
EndProcedure

Procedure.d Calc_Expression(Array TokenArray.Calc_Tokens(1))
   Protected op.s
   Protected v.d, r.d

   v = Calc_SingleExpression(TokenArray())
   op = TokenArray(Calc_GetToken(0))\Str
   While FindString(#Calc_OpRel, op, 1) <> 0
      Calc_GetToken(1)
      r = Calc_SingleExpression(TokenArray())
      v = Calc_Arith(op, v, r)
      op = TokenArray(Calc_GetToken(0))\Str
   Wend
   ProcedureReturn v
EndProcedure

;-----------------------------------------------------------------------

Procedure.s Calc_MyStr (v.d)
   ;-- Hilfsroutine, die das Ergebnis von StrD() ohne berflssige
   ;   Nullen nach dem Dezimaltrennzeichen liefert
   Protected ret.s
   Protected p.l

   ret = StrD(v)
   p = Len(ret)
   While Mid(ret, p, 1) = "0"
      p - 1
   Wend
   If Mid(ret, p, 1) = "."
      p - 1
   EndIf
   ProcedureReturn Left(ret, p)
EndProcedure

Procedure.s Calc (expr.s,*v.d=#Null,*err.i=#Null)
   ; * Hauptfunktion *
   ; in : zu berechnender Ausdruck als String
   ; out: Ergebnis als String (1. Zeichen ist eine Ziffer)
   ;               oder
   ;      Fehlermeldung (1. Zeichen ist ein Buchstabe)
   Protected v.d
   Protected err.i = 0
   Protected Dim TokenArray.Calc_Tokens(0)
   Protected Token.s

   Calc_Scan(LCase(expr),TokenArray())
   Calc_GetToken(0,0)
   Calc_ErrorI(0)
   v = Calc_Expression(TokenArray())

   If *v<>#Null
     PokeD(*v,v)
   EndIf
   err = Calc_ErrorI()
   If *err<>#Null
     PokeI(*err,err)
   EndIf
   Token = TokenArray(Calc_GetToken(0))\Str
   If err
      ProcedureReturn Calc_GetErrorText(err)
   ElseIf Token = ")"
      ProcedureReturn Calc_GetErrorText(#CalcErr_NoOpeningBracket)
   ElseIf Token = ","
      ProcedureReturn Calc_GetErrorText(#CalcErr_BadSyntax)
   Else                                ; kein Fehler
      ProcedureReturn Calc_MyStr(v)         ; Ergebnis zurckgeben
   EndIf
EndProcedure

Procedure.s Calc_FktStr(dWert.d,*dFkt.d=#Null,*iErr.i=#Null,sVar.s="",sFkt.s="")
  Static sVarL.s = ""
  Static sFktL.s = ""
  Protected sWert.s, sCalcFkt.s
  Protected iPow.i = Int(Log10(Abs(dWert)))
  Protected iErr.i
  Protected dFkt.d
  
  If sVar <> ""
    sVarL = sVar
  EndIf
  If sFkt <> ""
    sFktL = sFkt
  EndIf
    
  If iPow>-3 And iPow<7
    sWert = StrD(dWert,16-iPow)
  Else
    sWert = StrD(dWert*Pow(10.,-iPow))+"e"+Str(iPow)
  EndIf
  sWert = "("+sWert+")"
  
  sCalcFkt = ReplaceString(sFktL,sVarL,sWert)
  
  sWert = Calc(sCalcFkt,@dFkt,@iErr)
  
  If *dFkt <> #Null
    PokeD(*dFkt,dFkt)
  EndIf
  If *iErr <> #Null
    PokeI(*iErr,iErr)
  EndIf
  If iErr=#CalcErr_NegativeBase Or iErr=#CalcErr_DivisionByZero Or iErr=#CalcErr_PowerUndefined Or iErr=#CalcErr_Log Or iErr=#CalcErr_LogBase Or iErr=#CalcErr_OutOfPrecition Or iErr=#CalcErr_Tan Or iErr=#CalcErr_ACosASin
    ProcedureReturn ""
  ElseIf iErr
    ProcedureReturn Calc_GetErrorText(iErr)
  EndIf
  
  ProcedureReturn sWert
  
EndProcedure

Procedure.d Calc_FktDbl(dWert.d)
  
  Protected dFkt.d
  Protected iErr.i
  Protected sWert.s = Calc_FktStr(dWert,@dFkt,@iErr)
  
  If iErr=#CalcErr_NegativeBase Or iErr=#CalcErr_DivisionByZero Or iErr=#CalcErr_PowerUndefined Or iErr=#CalcErr_Log Or iErr=#CalcErr_LogBase Or iErr=#CalcErr_OutOfPrecition Or iErr=#CalcErr_Tan Or iErr=#CalcErr_ACosASin
    ProcedureReturn 0.0
  ElseIf iErr
    ProcedureReturn 0.0
  EndIf
  
  ProcedureReturn dFkt
  
EndProcedure

; *********************************************************************
; * Die Funktion realisiert ein Iterationsverfahren                   *
; * zur Lsung der Gleichung F(x) = 0.                                *
; * Die Funktion F mussParameter bergeben werden.                    *
; *********************************************************************
; * Anwendung:                                                        *
; * =========                                                         *
; * Bestimmung von Nullstellen der stetig differenzierbaren Funktion  *
; * Dabei muss entweder ein geeigneter Startwert oder 2 die           *
; * Nullstelle einschliessende Werte fuer die Iteration vorgegeben    *
; * werden.                                                           *
; *********************************************************************
; * Eingabe:                                                          *
; * ========                                                          *
; * *F.Pb_DblFkt   Funktion, deren Nullstelle zu bestimmen ist        *
; * *dX1.d         iModus = 0 Startwert der Iteration                 *
; *                iModus = 1,2 Intervallgrenze                       *
; * *dX2.d         iModus = 0 #Null                                   *
; *                iModus = 1,2 Intervallgrenze                       *
; * *iIter.i       vorgegebene Anzahl der Iterationen oder #Null      *
; * *dFktF.d       max. Gre des Funktionswerte oder #Null           *
; * *dRelF.d       Relativer Fehler oder #Null                        *
; * *dAbsF.d       Absoluter Fehler oder #Null                        *
; *                                                                   *
; * Ausgabe:                                                          *
; * ========                                                          *
; * *dX1.d         Nherung fr die Nullstelle                        *
; * *dY.d          Funktionswert der Nherungslsung                  *
; * *iIter.i       Anzahl der durchgefhrten Iterationsschritte       *
; * Funktionswert:                                                    *
; * ==============                                                    *
; * 0         Nullstelle mit |*dY| < *dFktF gefunden                  *
; * 1         Abbruch mit |dX| < *dAbsF + pdX1 * *dRelF               *
; * 2         zu viele Iterationen                                    *
; * 3         falsche Eingaben                                        *
; * 4         Kein Einschluss der Nullstelle zwischen den             *
; *           Anfangswerten                                           *
; *********************************************************************
Procedure.i PbNum_Nullstelle (*F.Pb_DblFkt,*dX1.d,*dX2.d,*dY.d,*iIter.i=#Null,*dFktF.d=#Null,*dRelF.d=#Null,*dAbsF=#Null)

  Protected iRet.i=1, iIter.i=0, iIterMax.i=300
  Protected dFktFehl.d=0.0, dAbsFehl.d=0.0, dRelFehl.d=0.0
  Protected dTol.d, dXm.d, dD.d=0.0, dE.d=0.0, dP.d, dQ.d, dR.d, dTmp.d, dX1.d, dX2.d, dX3.d, dY1.d, dY2.d, dY3.d, dS.d

  
  If *iIter <> #Null
    iIterMax = PeekI(*iIter)
  EndIf

  If *F = #Null
    ProcedureReturn 3
  EndIf
  
  dX1 = PeekD(*dX1)
  dX2 = PeekD(*dX2)
  dX3 = 0.0

  dY1 = *F(dX1) 
  dY2 = *F(dX2) 
  iIter = 0

  dTmp = dY1 * dY2
  If dTmp > 0.0
    ProcedureReturn -1
  ElseIf dTmp = 0.0
    ProcedureReturn 2
  EndIf

  If *dRelF = #Null
    dRelFehl = 4 * #PB_DBL_EPS
  Else
    dRelFehl = PeekD(*dRelF)
  EndIf
  
  If *dFktF = #Null
    dFktFehl = #PB_EPS_QUAD
  Else
    dFktFehl = PeekD(*dFktF)
  EndIf

  If dAbsFehl < 0.0 Or dRelFehl < 0.0 Or dAbsFehl + dRelFehl < 0.0 Or iIterMax < 1
    ProcedureReturn 3
  EndIf
  If dRelFehl = 0.0 And dAbsFehl < #PB_DBL_EPS
    dAbsFehl = #PB_DBL_EPS
  ElseIf dAbsFehl = 0.0 And dRelFehl < #PB_DBL_EPS
    dRelFehl = #PB_DBL_EPS
  Else
    If dAbsFehl < #PB_DBL_EPS
      dAbsFehl = #PB_DBL_EPS
    EndIf
    If dRelFehl < #PB_DBL_EPS
      dRelFehl = #PB_DBL_EPS
    EndIf
  EndIf

  dY3 = dY2
  Repeat
    If dY2 * M_SIGN(dY3) > 0.0
      dX3 = dX1
      dY3 = dY1
      dD  = dX2 - dX1
      dE  = dD
    EndIf
    If M_ABS(dY3) < M_ABS(dY2)
      dX1 = dX2
      dX2 = dX3
      dX3 = dX1
      dY1 = dY2
      dY2 = dY3
      dY3 = dY1
    EndIf 
    dTol = 0.5 * (dAbsFehl + dRelFehl * M_ABS(dX2))
    dXm  = 0.5 * (dX3 - dX2)

    If M_ABS(dXm) <= dTol
      If M_ABS(dY2) < dFktFehl
        iRet = 0
      Else
        iRet = 1
      EndIf
      Break
    EndIf
    If dY2 = 0.0
      iRet = 0
      Break
    EndIf

    dR = 0.0

    If M_ABS(dE) < dTol Or M_ABS(dY1) <= M_ABS(dY2)
      dD = dXm
      dE = dXm
    Else
      If dX1 <> dX3
        dQ = dY1 / dY3
        dR = dY2 / dY3
        dS = dY2 / dY1
        dP = dS * (2.0 * dXm * dQ * (dQ - dR) - (dX2 - dX1) * (dR - 1.0))
        dQ = (dQ - 1.0) * (dR - 1.0) * (dS - 1.0)
      Else
        dS = dY2 / dY1
        dP = 2.0 * dXm * dS
        dQ = 1.0 - dS
      EndIf
      If dP > 0.0
        dQ = -dQ
      Else
        dP = M_ABS(dP)
      EndIf
      If 2.0 * dP >= 3.0 * dXm * dQ - M_ABS(dTol * dQ) Or dP >= M_ABS(0.5 * dE * dQ)
        dE = dXm
        dD = dXm
      Else
        dE = dD
        dD = dP / dQ
      EndIf
    EndIf

    dX1 = dX2
    dY1 = dY2

    If M_ABS(dD) > dTol
      dX2 + dD
    Else
      dX2 + M_SIGN2(dTol, dXm)
    EndIf

    dY2 = *F(dX2)
    iIter + 1

    If iIter > iIterMax
      iRet = 3
      Break
    EndIf
  ForEver
  PokeD(*dY,dY2)
  PokeD(*dX1,dX2)
  PokeD(*dX2,dX1)
  If *dAbsF <> #Null
    PokeD(*dAbsF,dAbsFehl)
  EndIf
  If *dRelF <> #Null
    PokeD(*dRelF,dRelFehl)
  EndIf
  If *iIter <> #Null
    PokeI (*iIter,iIter)
  EndIf
  
  ProcedureReturn iRet
EndProcedure

Procedure DrawFunction (sFkt.s,Array dXY.d(1),Array iXY.i(1))
  Protected dSkalX.d = (dXY(1)-dXY(0))/(iXY(1)-iXY(0))
  Protected dSkalY.d = (iXY(3)-iXY(2))/(dXY(3)-dXY(2))
  Protected dDiff.d  = dSkalX*0.01,dX1.d = dXY(0),dY1.d,dDY1.d,dY2.d,dDY2.d,dDY.d,dX2.d
  Protected i.i,iErr1.i,iErr.i,iErr2.i
  
  Calc_FktStr(0.0,#Null,#Null,"x",sFkt)

  Calc_FktStr(dX1,@dY1,@iErr1)
  For i=iXY(0)+1 To iXY(1) Step 1
    
    Calc_FktStr(dX1+dDiff,@dDY1,@iErr)
    dX2 = dX1 + dSkalX
    Calc_FktStr(dX2-dDiff,@dDY2,@iErr)
    Calc_FktStr(dX2,@dY2,@iErr2)
    dDY = (dY2-dY1)/Abs(dSkalX) 
    dDY1 = (dDY1-dY1)/Abs(dDiff)
    dDY2 = (dY2-dDY2)/Abs(dDiff)
    If iErr1 Or iErr2 Or (((dDY1<dDY And dDY2<dDY) Or (dDY1>dDY And dDY2>dDY)) And (M_ABS(dDY1-dDY)>4*#PB_EPS_ROOT And M_ABS(dDY2-dDY)>4*#PB_EPS_ROOT))
      dX1 = dX2
      dY1 = dY2    
      iErr1 = iErr2
      Continue
    Else  
    
    EndIf
    
    LineXY(i-1,Int((dY1-dXY(2))*dSkalY)+iXY(2),i,Int((dY2-dXY(2))*dSkalY)+iXY(2))
    
    dX1 = dX2
    dY1 = dY2
    iErr1 = iErr2
  Next i
  
EndProcedure

Procedure BerechneNullstellen(sFkt1.s,sFkt2.s)

  Protected sFkt.s = sFkt1
  
  If sFkt2 <> ""
    sFkt + "-(" + sFkt2 + ")"
  EndIf

  If sFkt = ""
    ProcedureReturn 0
  EndIf
  
  Protected iErr.i = 0
  Protected Dim dXY.d(4) 
  Protected Dim iXY.i(4)
  dXY(0) = ValD(GetGadgetText(#String_Xmin))
  dXY(1) = ValD(GetGadgetText(#String_Xmax))
  dXY(2) = ValD(GetGadgetText(#String_Ymin))
  dXY(3) = ValD(GetGadgetText(#String_Ymax))
  iXY(0) = 0
  iXY(1) = GadgetWidth(#Image_Ausgabe)
  iXY(2) = GadgetHeight(#Image_Ausgabe)
  iXY(3) = 0
  Protected dSkalX.d = (dXY(1)-dXY(0))/(iXY(1)-iXY(0))
  Protected dSkalY.d = (iXY(3)-iXY(2))/(dXY(3)-dXY(2))
  Protected dDiff.d  = dSkalX*0.01,dX1.d = dXY(0),dY1.d,dDY1.d,dY2.d,dDY2.d,dDY.d,dX2.d,dFehl.d=0.0000000000001
  Protected i.i,iErr1.i,iErr2.i

  If GetGadgetText(#String_Xmin) = "" And GetGadgetText(#String_Xmax) = "" And GetGadgetText(#String_Ymin) = "" And GetGadgetText(#String_Ymax) = ""
    ProcedureReturn 0
  EndIf
  
  sFkt = LCase(sFkt)
  Protected sErr.s = Calc_FktStr(0.0,#Null,#Null,"x",sFkt)
  If sErr <> "" And Left(sErr,1) <> "-" And (Asc(sErr)<48 Or Asc(sErr)>57)
    iErr = 1
    MessageRequester("Achtung!!!", "eine der Funktionen ist falsch definiert",#PB_MessageRequester_Ok)
    ProcedureReturn 0
  EndIf

  Calc_FktStr(0.0,#Null,#Null,"x",sFkt)
  Calc_FktStr(dX1,@dY1,@iErr1)
  For i=iXY(0)+1 To iXY(1) Step 1
    
    Calc_FktStr(dX1+dDiff,@dDY1,@iErr)
    dX2 = dX1 + dSkalX
    Calc_FktStr(dX2-dDiff,@dDY2,@iErr)
    Calc_FktStr(dX2,@dY2,@iErr2)
    dDY = (dY2-dY1)/Abs(dSkalX)
    dDY1 = (dDY1-dY1)/Abs(dDiff)
    dDY2 = (dY2-dDY2)/Abs(dDiff)
    If iErr1 Or iErr2 Or (((dDY1<dDY And dDY2<dDY) Or (dDY1>dDY And dDY2>dDY)) And (M_ABS(dDY1-dDY)>M_ABS(dSkalX) And M_ABS(dDY2-dDY)>M_ABS(dSkalX)))
      dX1 = dX2
      dY1 = dY2    
      iErr1 = iErr2
      Continue
    EndIf

    If dY1*dY2 < 0.0
      Protected dX11.d = dX1 ,dX21.d = dX2, dY.d
      Protected iIter = 200    
      iErr = PbNum_Nullstelle (@Calc_FktDbl(),@dX11.d,@dX21.d,@dY.d,@iIter,@dFehl)
      If iErr = 0 And Abs(dY)<dFehl
        AddGadgetItem(#ListIcon_Schnittpunkt,-1,"")
        SetGadgetItemText(#ListIcon_Schnittpunkt,CountGadgetItems(#ListIcon_Schnittpunkt)-1,StrD(dX11),0)
      EndIf
    ElseIf dY1 = 0.0 And i=iXY(0)+1
      AddGadgetItem(#ListIcon_Schnittpunkt,-1,"")
      SetGadgetItemText(#ListIcon_Schnittpunkt,CountGadgetItems(#ListIcon_Schnittpunkt)-1,StrD(dX1),0)
    ElseIf dY2 = 0.0
      AddGadgetItem(#ListIcon_Schnittpunkt,-1,"")
      SetGadgetItemText(#ListIcon_Schnittpunkt,CountGadgetItems(#ListIcon_Schnittpunkt)-1,StrD(dX2),0)
    EndIf

    dX1 = dX2
    dY1 = dY2
    iErr1 = iErr2
  Next i

  Calc_FktStr(0.0,#Null,#Null,"x",sFkt1)
  For i=0 To CountGadgetItems(#ListIcon_Schnittpunkt)-1 Step 1
    dX1 = ValD(GetGadgetItemText(#ListIcon_Schnittpunkt,i,0))
    dY1 = Calc_FktDbl(dX1)
    SetGadgetItemText(#ListIcon_Schnittpunkt,i,StrD(dY1),1)
  Next i

  ProcedureReturn 1
EndProcedure

;******************************************************************************
; F: Zeichen einer Skala, StartDrawing() muss bereits aufgerufen worden sein
; E: iRicht.i                0 - X
;                            1 - Y
;    iVon.i,iBis.i           Zeichenbereich
;    dVon.d,dBis.d           Wertebereich
;    iTypText.i              Nachkommastellen
;    iStyle.i                #BIT_0 = 1 Skalenstriche oben/links von der 
;                                       Linie beginnen
;                            #BIT_1 = 1 Skalenstriche unten/rechte von der 
;                                       Linie beenden
;                            #BIT_2 = 1 Text oberhalb der Linie
;                            #BIT_3 = 1 Text bei Y links ausrichten
;                            #BIT_4 = 1 Wert(Sekunden) in Minuten und Sekunden 
;                                       ausgeben
;    dStepText.d             Schrittweite im Wertebereich fr Skalenbeschriftung
;    iAnzZw.i                Anzahl Zwischenwerte
;    iLSt1.i                 Lnge kurze Skalenstriche
;    iLSt2.i                 Lnge lange Skalenstriche
;    iColText.i              Textfarbe
;    iColLine.i              Linienfarbe
; R: Skalierungswert der Achse
;*******************************************************************************
Procedure.d DrawSkala(iRicht.i,iVon.i,iBis.i,iPos.i,dVon.d,dBis.d,iTypText.i,iStyle.i,dStepText.d,iAnzZw.i,iLSt1.i,iLSt2.i,iColText.i,iColLine.i)
  Protected dSkal.d = (iBis-iVon)/(dBis-dVon), dWert.d
  Protected iLSt11.i,iLSt12.i,iLSt21.i,iLSt22.i,i.i,iStart.i,iEnd.i,iPos1.i,iMaxTx.i,iMaxTy.i,iLastPos.i,iPos2.i
  Protected sWert.s
  
  If (iStyle&#BIT_0) <> 0
    iLSt11.i = iPos-iLSt1
    iLSt21.i = iPos-iLSt2
  Else
    iLSt11.i = iPos
    iLSt21.i = iPos
  EndIf
  If (iStyle&#BIT_1) <> 0
    iLSt12.i = iPos+iLSt1
    iLSt22.i = iPos+iLSt2
  Else
    iLSt12.i = iPos
    iLSt22.i = iPos
  EndIf
  
  If iAnzZW > 1
    Protected dAbstZw.d = dStepText/iAnzZw*dSkal
    While Abs(dAbstZw) < 4 And iAnzZW > 1
      iAnzZw / 2
      dAbstZw = dStepText/iAnzZw*dSkal
    Wend
  EndIf
  ; Grundlinie
  If iRicht = 0
    LineXY(iVon,iPos,iBis,iPos,iColLine)
  Else  
    LineXY(iPos,iVon,iPos,iBis,iColLine)
  EndIf
  ; kleine Markierungen
  If iAnzZW > 1
    iStart = Int(dVon/dAbstZW*dSkal)
    iEnd = Int(dBis/dAbstZW*dSkal)
    For i=iStart To iEnd Step 1
      iPos1 = iVon + Int(i*dAbstZW-dVon*dSkal)
      If iRicht = 0
        LineXY(iPos1,iLSt11,iPos1,iLSt12,iColLine)
      Else
        LineXY(iLSt11,iPos1,iLSt12,iPos1,iColLine)
      EndIf
    Next i
  EndIf
  ; groe Markierungen
  iStart = Int(dVon/dStepText)
  iEnd = Int(dBis/dStepText)
  For i=iStart To iEnd Step 1
    iPos1 = iVon + Int((i*dStepText-dVon)*dSkal)
    If iRicht = 0
      LineXY(iPos1,iLSt21,iPos1,iLSt22,iColLine)
    Else
      LineXY(iLSt21,iPos1,iLSt22,iPos1,iColLine)
    EndIf
  Next i
  ; Textausma
  iMaxTx = 0
  iMaxTy = 0
  For i=iStart To iEnd Step 1
    dWert = i*dStepText
    If (iStyle&#BIT_4) <> 0 ; Zeitwert in Sekunden
      sWert = Str(Int(dWert/60))+":"+StrD(dWert-Int(dWert/60)*60.0,2)
    Else ; normaler Double-Wert
      sWert = StrD(dWert,iTypText)
    EndIf
    iPos1 = TextWidth(sWert)
    If iPos1 > iMaxTx
      iMaxTx = iPos1
    EndIf
    iPos1 = TextHeight(sWert)
    If iPos1 > iMaxTy
      iMaxTy = iPos1
    EndIf
  Next i
  ; Textausgabe
  DrawingMode(#PB_2DDrawing_Transparent)
  iLastPos = iVon
  For i=iStart To iEnd Step 1
    iPos1 = iVon + Int((i*dStepText-dVon)*dSkal)
    dWert = i*dStepText
    If (iStyle&#BIT_4) <> 0 ; Zeitwert in Sekunden
      sWert = Str(Int(dWert/60))+":"+RSet(StrD(dWert-Int(dWert/60)*60.0,2),5,"0")
    Else ; normaler Double-Wert
      sWert = StrD(dWert,iTypText)
    EndIf
    If iRicht = 0
      If (iStyle&#BIT_2) <> 0
        iPos2 = iLSt21-iMaxTy
      Else
        iPos2 = iLSt22
      EndIf
      If iPos1-TextWidth(sWert)/2 > iLastPos+1
        DrawText(iPos1-TextWidth(sWert)/2,iPos2,sWert,iColText)
      EndIf
      iLastPos = iPos1 + TextWidth(sWert)/2 + 1
    Else
      If (iStyle&#BIT_2) <> 0
        iPos2 = iLSt22
      Else
        If (iStyle&#BIT_3) <> 0
          iPos2 = iLSt21-iMaxTx
        Else
          iPos2 = iLSt21-TextWidth(sWert)
        EndIf        
      EndIf
      If iVon < iBis
        If iPos1-TextHeight(sWert)/2 > iLastPos+1
          DrawText(iPos2,iPos1+TextHeight(sWert)/2,sWert,iColText)
        EndIf
        iLastPos = iPos1 + TextHeight(sWert)/2 + 1
      Else
        If iPos1+TextHeight(sWert)/2 < iLastPos-1
          DrawText(iPos2,iPos1-TextHeight(sWert)/2,sWert,iColText)
        EndIf
        iLastPos = iPos1 - TextHeight(sWert)/2 - 1      
      EndIf
    EndIf
  Next i
  DrawingMode(#PB_2DDrawing_Default)
  ProcedureReturn dSkal
EndProcedure

Procedure Open_Window_Num()
  If OpenWindow(#Window_Num, 413, 305, 560, 515, "Schnittpunkt",  #PB_Window_SystemMenu | #PB_Window_SizeGadget | #PB_Window_TitleBar )
    If CreateStatusBar(#StatusBar_Fenster, WindowID(#Window_Num))
      AddStatusBarField(550)
      StatusBarText(#StatusBar_Fenster, 0, "Programm erfolgreich gestartet.")
    EndIf

    ;If CreateGadgetList(WindowID(#Window_Num))
      ImageGadget(#Image_Ausgabe, 5, 5, 545, 250, #Image0)
      TextGadget(#Text_Form1, 10, 270, 75, 15, " y1 = f1 ( x ) =")
      TextGadget(#Text_Form2, 10, 295, 75, 15, " y2 = f2 ( x ) =")
      StringGadget(#String_Form1, 90, 265, 200, 20, "")
      StringGadget(#String_Form2, 90, 290, 200, 20, "")
       
      ListIconGadget(#ListIcon_Schnittpunkt, 10, 325, 280, 155, "Schnittpunkt x", 137,#PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect)
      AddGadgetColumn(#ListIcon_Schnittpunkt, 1, "Schnittpunkt y", 137)
      TextGadget(#Text_Xmin, 320, 270, 65, 15, " X min")
      TextGadget(#Text_Xmax, 320, 295, 65, 15, " X max")
      TextGadget(#Text_Ymin, 320, 320, 65, 15, " Y min")
      TextGadget(#Text_Ymax, 320, 345, 65, 15, " Y max")
      StringGadget(#String_Xmin, 390, 265, 155, 20, "")
      StringGadget(#String_Xmax, 390, 290, 155, 20, "")
      StringGadget(#String_Ymin, 390, 315, 155, 20, "")
      StringGadget(#String_Ymax, 390, 340, 155, 20, "")
      ButtonGadget(#Button_Berechnen, 315, 380, 235, 30, "Darstellen", #PB_Button_Default)
      ButtonGadget(#Button_Beenden, 315, 450, 235, 30, "BEENDEN")
      ButtonGadget(#Button_Info, 315, 415, 235, 30, "Schnittpunkte/Nullstellen")
    ;EndIf

    Protected lsDesk.RECT
    SystemParametersInfo_(#SPI_GETWORKAREA,0,@lsDesk.RECT,0)
    WindowBounds(#Window_Num, 560, 515, lsDesk\right - lsDesk\left,lsDesk\bottom - lsDesk\top)
    
    StartDrawing(ImageOutput(#Image0))
    Box(0,0,545,250,RGB(50,50,50))
    StopDrawing()
    SetGadgetState(#Image_Ausgabe,ImageID(#Image0))
    ProcedureReturn 1
  Else
    ProcedureReturn 0
  EndIf
  
EndProcedure

Procedure WindowAnpassen()

  Protected iWinX = WindowWidth(#Window_Num)
  Protected iWinY = WindowHeight(#Window_Num)

  ResizeGadget(#Image_Ausgabe,#PB_Ignore,#PB_Ignore,iWinX-10, iWinY - GadgetHeight(#ListIcon_Schnittpunkt)-StatusBarHeight(#StatusBar_Fenster)-GadgetHeight(#String_Form1)-GadgetHeight(#String_Form2)-45)
  ResizeGadget(#ListIcon_Schnittpunkt,#PB_Ignore, iWinY - GadgetHeight(#ListIcon_Schnittpunkt) - StatusBarHeight(#StatusBar_Fenster)-15 , iWinX/2-45,#PB_Ignore)
  SetGadgetItemAttribute(#ListIcon_Schnittpunkt,0,#PB_ListIcon_ColumnWidth,GadgetWidth(#ListIcon_Schnittpunkt)/2-2,0)
  SetGadgetItemAttribute(#ListIcon_Schnittpunkt,0,#PB_ListIcon_ColumnWidth,GadgetWidth(#ListIcon_Schnittpunkt)/2-2,1)
  
  ResizeGadget(#Text_Form1, #PB_Ignore,GadgetY(#Image_Ausgabe)+GadgetHeight(#Image_Ausgabe)+15, #PB_Ignore, #PB_Ignore)
  ResizeGadget(#String_Form1, #PB_Ignore, GadgetY(#Text_Form1)-5,iWinX/2-60-GadgetWidth(#Text_Form1),#PB_Ignore)
  ResizeGadget(#Text_Form2, #PB_Ignore, GadgetY(#Text_Form1)+GadgetHeight(#Text_Form1)+5,#PB_Ignore,#PB_Ignore)
  ResizeGadget(#String_Form2, #PB_Ignore, GadgetY(#Text_Form2)-5,iWinX/2-60-GadgetWidth(#Text_Form2),#PB_Ignore)
        
  ResizeGadget(#Text_Xmin, GadgetX(#String_Form1)+GadgetWidth(#String_Form1)+30,GadgetY(#Text_Form1),#PB_Ignore,#PB_Ignore)
  ResizeGadget(#Text_Xmax, GadgetX(#Text_Xmin),GadgetY(#Text_Xmin)+GadgetHeight(#Text_Xmin)+10,#PB_Ignore,#PB_Ignore)
  ResizeGadget(#Text_Ymin, GadgetX(#Text_Xmin),GadgetY(#Text_Xmax)+GadgetHeight(#Text_Xmax)+10,#PB_Ignore,#PB_Ignore)
  ResizeGadget(#Text_Ymax, GadgetX(#Text_Xmin),GadgetY(#Text_Ymin)+GadgetHeight(#Text_Ymin)+10,#PB_Ignore,#PB_Ignore)
  ResizeGadget(#String_Xmin, GadgetX(#Text_Xmin)+GadgetWidth(#Text_Xmin)+15,GadgetY(#Text_Xmin)-5,iWinX-GadgetX(#Text_Xmin)-GadgetWidth(#Text_Xmin)-25 , #PB_Ignore)
  ResizeGadget(#String_Xmax, GadgetX(#String_Xmin),GadgetY(#Text_Xmax)-5,GadgetWidth(#String_Xmin), #PB_Ignore)
  ResizeGadget(#String_Ymin, GadgetX(#String_Xmin),GadgetY(#Text_Ymin)-5,GadgetWidth(#String_Xmin), #PB_Ignore)
  ResizeGadget(#String_Ymax, GadgetX(#String_Xmin),GadgetY(#Text_Ymax)-5,GadgetWidth(#String_Xmin), #PB_Ignore)
  
  ResizeGadget(#Button_Berechnen, GadgetX(#Text_Xmin)-5,GadgetY(#String_Ymax)+GadgetHeight(#Text_Ymax)+20, GadgetWidth(#Text_Xmin)+GadgetWidth(#String_Xmin)+10, #PB_Ignore)
  ResizeGadget(#Button_Info     , GadgetX(#Button_Berechnen),GadgetY(#Button_Berechnen)+GadgetHeight(#Button_Berechnen)+5, GadgetWidth(#Button_Berechnen), #PB_Ignore)
  ResizeGadget(#Button_Beenden  , GadgetX(#Button_Berechnen),GadgetY(#Button_Info)+GadgetHeight(#Button_Info)+5, GadgetWidth(#Button_Berechnen), #PB_Ignore)

  ResizeImage(#Image0,GadgetWidth(#Image_Ausgabe),GadgetHeight(#Image_Ausgabe))

EndProcedure

Procedure.i DrawAllFunctions()

  Protected Dim dXY.d(4) 
  Protected Dim iXY.i(4)
  Protected sFkt1.s
  Protected sFkt2.s

  If GetGadgetText(#String_Xmin) = "" And GetGadgetText(#String_Xmax) = "" And GetGadgetText(#String_Ymin) = "" And GetGadgetText(#String_Ymax) = ""
    ProcedureReturn (0)
  EndIf
  
  dXY(0) = ValD(GetGadgetText(#String_Xmin))
  dXY(1) = ValD(GetGadgetText(#String_Xmax))
  dXY(2) = ValD(GetGadgetText(#String_Ymin))
  dXY(3) = ValD(GetGadgetText(#String_Ymax))
  iXY(0) = 0
  iXY(1) = GadgetWidth(#Image_Ausgabe)
  iXY(2) = GadgetHeight(#Image_Ausgabe)
  iXY(3) = 0
  Protected iErr.i = 0

  Protected iLog.i = Int(Log10(dXY(1)-dXY(0))-1)
  Protected dStep.d = Pow(10.0,iLog)
  Protected iNach.i, iDiff.i, iStyle.i = #BIT_0|#BIT_1
  If iLog >= 0
    iNach = 0
  Else
    iNach = -iLog
  EndIf
  If iXY(2) > iXY(3)
    iDiff = -1
  Else
    iDiff = 1
  EndIf
  Protected iPos.i = iXY(2)+Int((dXY(2)/(dXY(2)-dXY(3)))*(iXY(3)-iXY(2)))
  If iPos*iDiff > iXY(3)*iDiff
    iPos = iXY(3)
  ElseIf iPos*iDiff < iXY(2)*iDiff
    iPos = iXY(2)
  EndIf
 
  DrawSkala(0,iXY(0),iXY(1),iPos,dXY(0),dXY(1),iNach,iStyle,dStep,10,2,4,RGB(100,100,100),RGB(120,120,120))

  iLog = Int(Log10(dXY(3)-dXY(2))-0.1)
  dStep = Pow(10.0,iLog)
  iStyle.i = #BIT_0|#BIT_1
  If iLog >= 0
    iNach = 0
  Else
    iNach = -iLog
  EndIf
  If iXY(0) > iXY(1)
    iDiff = -1
  Else
    iDiff = 1
  EndIf
  iPos.i = iXY(0)+Int((dXY(0)/(dXY(0)-dXY(1)))*(iXY(1)-iXY(0)))
  If iPos*iDiff > iXY(1)*iDiff
    iPos = iXY(1)
  ElseIf iPos*iDiff < iXY(0)*iDiff
    iPos = iXY(0)
  EndIf
 
  DrawSkala(1,iXY(2),iXY(3),iPos,dXY(2),dXY(3),iNach,iStyle,dStep,10,2,4,RGB(100,100,100),RGB(120,120,120))

  If dXY(0) >= dXY(1) Or dXY(2) >= dXY(3)
    MessageRequester("Achtung!!!", "Bereich falsch definiert",#PB_MessageRequester_Ok)
  Else
    sFkt1.s = GetGadgetText(#String_Form1)
    sFkt2.s = GetGadgetText(#String_Form2)
    
    If sFkt1="" And sFkt2=""
      MessageRequester("Achtung!!!", "Mindestens eine Funktion muss definiert sein",#PB_MessageRequester_Ok)
    Else
      If sFkt1<>""
        sFkt1 = LCase(sFkt1)
        Protected sErr.s = Calc_FktStr(0.0,#Null,#Null,"x",sFkt1)
        If sErr <> "" And Asc(sErr) <> 45 And (Asc(sErr)<48 Or Asc(sErr)>57)
          iErr = 1
          MessageRequester("Achtung!!!", "Funktion 1 ist falsch definiert: "+sErr,#PB_MessageRequester_Ok)
        EndIf
      EndIf
      If sFkt2<>""
        sFkt2 = LCase(sFkt2)
        sErr = Calc_FktStr(0.0,#Null,#Null,"x",sFkt2)
        If sErr <> "" And Asc(sErr) <> 45 And (Asc(sErr)<48 Or Asc(sErr)>57)
          iErr = 1
          MessageRequester("Achtung!!!", "Funktion 2 ist falsch definiert: "+sErr,#PB_MessageRequester_Ok)
        EndIf
      EndIf
    EndIf
  EndIf
  If iErr = 0
    If sFkt1 <> ""
      FrontColor(RGB(255,0,0))
      DrawFunction (sFkt1.s,dXY(),iXY())
    EndIf
    If sFkt2 <> ""
      FrontColor(RGB(0,255,0))
      DrawFunction (sFkt2.s,dXY(),iXY())
    EndIf
    SetGadgetState(#Image_Ausgabe,ImageID(#Image0))
  EndIf        

  ProcedureReturn iErr
EndProcedure

Procedure.i DrawSchnittpunkt( dX.d, dY.d)

  Protected Dim dXY.d(4) 
  Protected Dim iXY.i(4)

  dXY(0) = ValD(GetGadgetText(#String_Xmin))
  dXY(1) = ValD(GetGadgetText(#String_Xmax))
  dXY(2) = ValD(GetGadgetText(#String_Ymin))
  dXY(3) = ValD(GetGadgetText(#String_Ymax))
  iXY(0) = 0
  iXY(1) = GadgetWidth(#Image_Ausgabe)
  iXY(2) = GadgetHeight(#Image_Ausgabe)
  iXY(3) = 0

  If dXY(0) = dXY(1) Or dXY(2) = dXY(3) Or iXY(0) = iXY(1) Or iXY(2) = iXY(3)
    ProcedureReturn 1
  EndIf
  Protected dSkalX.d = (iXY(1)-iXY(0))/(dXY(1)-dXY(0))
  Protected dSkalY.d = (iXY(3)-iXY(2))/(dXY(3)-dXY(2))
  
  Circle (Int((dX-dXY(0))*dSkalX)+iXY(0),Int((dY-dXY(2))*dSkalY)+iXY(2),2,RGB(255,255,0))

  ProcedureReturn 0
EndProcedure

Procedure Main()

  If CreateImage(#Image0,545,250) = 0
    MessageRequester("Achtung!","Kann Image nicht erstellen")
    End
  EndIf

  If Open_Window_Num() = 0
    MessageRequester("Achtung!","Fenster kann nicht geffnet werden")
    End
  EndIf

  Repeat
    Protected iEvent.i = WaitWindowEvent()
     
    Select iEvent

      Case #PB_Event_SizeWindow
        Protected iAktiv.i = GetGadgetState(#ListIcon_Schnittpunkt)
        WindowAnpassen()
        StartDrawing(ImageOutput(#Image0))
        Box(0,0,GadgetWidth(#Image_Ausgabe),GadgetHeight(#Image_Ausgabe),RGB(50,50,50))
        DrawAllFunctions()
        If iAktiv <> -1
          DrawSchnittpunkt(ValD(GetGadgetItemText(#ListIcon_Schnittpunkt,iAktiv,0)),ValD(GetGadgetItemText(#ListIcon_Schnittpunkt,iAktiv,1)))
        EndIf
        StopDrawing()
        SetGadgetState(#Image_Ausgabe,ImageID(#Image0))

      Case #PB_Event_Gadget
      
        Select EventGadget()
          Case #Button_Beenden
            iEvent = #PB_Event_CloseWindow
          Case #Button_Berechnen
            ClearGadgetItems(#ListIcon_Schnittpunkt)
            StartDrawing(ImageOutput(#Image0))
            DrawAllFunctions()
            StopDrawing()
            SetGadgetState(#Image_Ausgabe,ImageID(#Image0))
          Case #Button_Info
            BerechneNullstellen (GetGadgetText(#String_Form1),GetGadgetText(#String_Form2))
          Case #String_Form1,#String_Form2,#String_Xmin,#String_Xmax,#String_Ymin,#String_Ymax
            If EventType() = #PB_EventType_Change
              ClearGadgetItems(#ListIcon_Schnittpunkt)
              StartDrawing(ImageOutput(#Image0))
              Box(0,0,GadgetWidth(#Image_Ausgabe),GadgetHeight(#Image_Ausgabe),RGB(50,50,50))
              StopDrawing()
              SetGadgetState(#Image_Ausgabe,ImageID(#Image0))
            EndIf
          Case #ListIcon_Schnittpunkt
            iAktiv.i = GetGadgetState(#ListIcon_Schnittpunkt)
            StartDrawing(ImageOutput(#Image0))
            Box(0,0,GadgetWidth(#Image_Ausgabe),GadgetHeight(#Image_Ausgabe),RGB(50,50,50))
            SetGadgetState(#Image_Ausgabe,ImageID(#Image0))
            DrawAllFunctions()
            If iAktiv <> -1
              DrawSchnittpunkt(ValD(GetGadgetItemText(#ListIcon_Schnittpunkt,iAktiv,0)),ValD(GetGadgetItemText(#ListIcon_Schnittpunkt,iAktiv,1)))
            EndIf
            StopDrawing()
            SetGadgetState(#Image_Ausgabe,ImageID(#Image0))
        EndSelect
    EndSelect
  Until iEvent = #PB_Event_CloseWindow
EndProcedure

Main()
; IDE Options = PureBasic 4.30 (Windows - x86)
; CursorPosition = 1235
; FirstLine = 1190
; Folding = -------
; EnableXP
; Watchlist = DrawFunction()>dX2;BerechneNullstellen()>dX1;BerechneNullstellen()>dY1
; Watchlist = BerechneNullstellen()>dX2;BerechneNullstellen()>dY2